/* linux/arch/arm/mach-msm/7x00-irq.c
  * Author: Martin Johnson
 * Based on irq.c by Brian Swetland <swetland@google.com>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
 
#include <linux/init.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/ptrace.h>
#include <linux/timer.h>

#include <linux/irq.h>
#include <asm/hardware.h>
#include <asm/irq.h>

#include <asm/io.h>

#include <asm/arch/msm_iomap.h>

#define VIC_REG(off) (MSM_VIC_BASE + (off))

// just a guess but it works

#define VIC_INT_0     	VIC_REG(0x0000)
#define VIC_INT_1       VIC_REG(0x0004)

#define VIC_INT_EN0     VIC_REG(0x0028)
#define VIC_INT_EN1     VIC_REG(0x002c)

#define VIC_INT_CLEAR0    VIC_REG(0x0018)
#define VIC_INT_CLEAR1    VIC_REG(0x001c)

#define VIC_INT_MASK0    VIC_REG(0x0040)
#define VIC_INT_MASK1    VIC_REG(0x0044)

// these are wrong
#define VIC_INT_POLARITY0    VIC_REG(0x0070)
#define VIC_INT_POLARITY1    VIC_REG(0x0074)

#define VIC_INT_TYPE0    VIC_REG(0x0070)
#define VIC_INT_TYPE1    VIC_REG(0x0074)


static uint32_t msm_irq_wake_enable[2];
static struct {
	uint32_t int_en;
	uint32_t int_type;
	uint32_t int_polarity;
} msm_irq_shadow_reg[2];

static void msm_irq_ack(unsigned int irq)
{
	unsigned reg = VIC_INT_CLEAR0 + ((irq & 32) ? 4 : 0);
	irq = 1 << (irq & 31);
	writel(irq, reg);
}

static void msm_irq_mask(unsigned int irq)
{
	unsigned reg = VIC_INT_MASK0 + ((irq & 32) ? 4 : 0);
	unsigned index = (irq >> 5) & 1;
	uint32_t mask = 1UL << (irq & 31);
	msm_irq_shadow_reg[index].int_en &= ~mask;
	writel(mask, reg);
}

static void msm_irq_unmask(unsigned int irq)
{
	unsigned reg = VIC_INT_EN0 + ((irq & 32) ? 4 : 0);
	unsigned index = (irq >> 5) & 1;
	uint32_t mask = 1UL << (irq & 31);
	msm_irq_shadow_reg[index].int_en |= mask;	
	writel(readl(reg) | mask, reg);
}

static int msm_irq_set_wake(unsigned int irq, unsigned int on)
{
	unsigned index = (irq >> 5) & 1;
	uint32_t mask = 1UL << (irq & 31);
	if (on)
		msm_irq_wake_enable[index] |= mask;
	else
		msm_irq_wake_enable[index] &= ~mask;
	return 0;
}

static int msm_irq_set_type(unsigned int irq, unsigned int flow_type)
{
	printk("set_type %d %d\n",irq,flow_type);
	unsigned treg = VIC_INT_TYPE0 + ((irq & 32) ? 4 : 0);
	unsigned preg = VIC_INT_POLARITY0 + ((irq & 32) ? 4 : 0);
	unsigned index = (irq >> 5) & 1;
	int b = 1 << (irq & 31);
	uint32_t polarity;
	uint32_t type;

	polarity = msm_irq_shadow_reg[index].int_polarity;
	if (flow_type & (IRQF_TRIGGER_FALLING | IRQF_TRIGGER_LOW))
		polarity |= b;
	if (flow_type & (IRQF_TRIGGER_RISING | IRQF_TRIGGER_HIGH))
		polarity &= ~b;
	//writel(polarity, preg);
	msm_irq_shadow_reg[index].int_polarity = polarity;

	type = msm_irq_shadow_reg[index].int_type;
	if (flow_type & (IRQF_TRIGGER_RISING)) { // | IRQF_TRIGGER_FALLING)) {
		type |= b;
	    //		irq_desc[irq].handle_irq = handle_edge_irq;
	}
	    /*
	if (flow_type & (IRQF_TRIGGER_HIGH | IRQF_TRIGGER_LOW)) {
		type &= ~b;
		irq_desc[irq].handle_irq = handle_level_irq;
	}
	    */
	writel(type, treg);
	msm_irq_shadow_reg[index].int_type = type;
	return 0;
	return 0;
}

void msm_irq_enter_sleep(bool arm9_wake)
{
	writel(0, VIC_INT_EN0);
	writel(0, VIC_INT_EN1);

	/*
	if (arm9_wake) {
  		msm_irq_set_type(INT_A9_M2A_6, IRQF_TRIGGER_RISING);
  		writel(1U << INT_A9_M2A_6, VIC_INT_EN0);
	} else {
   		writel(msm_irq_wake_enable[0], VIC_INT_EN0);
      writel(msm_irq_wake_enable[1], VIC_INT_EN1);
	}
	*/
	writel(1U<<3, VIC_INT_EN0); // enable modem irq
	writel(2,VIC_INT_EN1); // enable gpio irq
//	disable_irq(33+64);
}

void msm_irq_exit_sleep(void)
{
	int i;
	msm_irq_ack(INT_A9_M2A_6);
   for (i = 0; i < 2; i++) {
//   	writel(msm_irq_shadow_reg[i].int_type, VIC_INT_TYPE0 + i * 4);
//      writel(msm_irq_shadow_reg[i].int_polarity, VIC_INT_POLARITY0 + i * 4);
      writel(msm_irq_shadow_reg[i].int_en, VIC_INT_EN0 + i * 4);
//      writel(msm_irq_shadow_reg[i].int_select, VIC_INT_SELECT0 + i * 4);
   }
//	enable_irq(33+64);
}

static struct irq_chip msm_irq_chip = {
	.name      = "msm",
	.ack       = msm_irq_ack,
	.mask      = msm_irq_mask,
	.unmask    = msm_irq_unmask,
	.set_wake  = msm_irq_set_wake,
	.set_type  = msm_irq_set_type,
};

void __init msm_init_irq(void)
{
	unsigned n;
	/* disable all INTs */
	writel(0, VIC_INT_EN0);
	writel(0, VIC_INT_EN1);
	// clear any pending ints
	writel(-1, VIC_INT_CLEAR0);
	writel(-1, VIC_INT_CLEAR1);

	// set correct triggering for interrupts 0-7
	// edge or level, positive or nagative, who knows!
	writel(readl(VIC_INT_TYPE0) | 0xff,VIC_INT_TYPE0);
	//	writel(0xff,VIC_INT_TYPE0); 
	//	writel(0,VIC_INT_TYPE1); 
	msm_irq_shadow_reg[0].int_type=readl(VIC_INT_TYPE0);
	msm_irq_shadow_reg[1].int_type=readl(VIC_INT_TYPE1);

	/*
	writel(-1,VIC_INT_POLARITY0);
	writel(-1,VIC_INT_POLARITY1);
	*/
//	writel(-1,VIC_INT_TYPE0); // if set this triggers on rising otherwise rising and falling
//	writel(-1,VIC_INT_TYPE1);


	for (n = 0; n < NR_MSM_IRQS; n++) {
		set_irq_chip(n, &msm_irq_chip);
		set_irq_handler(n, handle_level_irq);
		set_irq_flags(n, IRQF_VALID);
	}
}
