/* linux/arch/arm/mach-msm/board-polaris.c
 *
 * Author: Martin Johnson, Lukas Gorris <lukas.gorris@gmail.com> , Waylon Grange <raven.coda@gmail.com>, Patrick Remy <patrick.remy2000@gmail.com>
 * Based on board-halibut.c by Brian Swetland <swetland@google.com>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/input.h>
#include <linux/i2c.h>
#include <linux/mfd/htc-egpio.h>
#include <linux/gpiodev2.h>

#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>
#include <asm/mach/mmc.h>

#include <asm/arch/board.h>
#include <asm/arch/clock.h>
#include <asm/arch/msm_iomap.h>
#include <asm/arch/msm_fb.h>
#include <asm/arch/vreg.h>

#include <asm/io.h>
#include <asm/delay.h>
#include <linux/delay.h>

#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>
#include <linux/gpio_event.h>

#include <asm/arch/gpio.h>


#define MSM_SMI_BASE		0x00000000
#define MSM_SMI_SIZE		0x900000

#define MSM_EBI_BASE		0x10000000
#define MSM_EBI_SIZE		0x6e00000

#define MSM_PMEM_GPU0_BASE	0x0
#define MSM_PMEM_GPU0_SIZE	0x800000

#define MSM_LINUX_BASE		MSM_EBI_BASE
#define MSM_LINUX_SIZE		0x4c00000

#define MSM_PMEM_MDP_BASE	MSM_LINUX_BASE + MSM_LINUX_SIZE
#define MSM_PMEM_MDP_SIZE	0x800000

#define MSM_PMEM_ADSP_BASE	MSM_PMEM_MDP_BASE + MSM_PMEM_MDP_SIZE
#define MSM_PMEM_ADSP_SIZE	0x800000

#define MSM_PMEM_GPU1_BASE	MSM_PMEM_ADSP_BASE + MSM_PMEM_ADSP_SIZE
#define MSM_PMEM_GPU1_SIZE	0x800000

#define MSM_FB_BASE		MSM_PMEM_GPU1_BASE + MSM_PMEM_GPU1_SIZE
#define MSM_FB_BASE  	 	0x00000000
#define MSM_FB_SIZE		0x9b000

#ifdef CONFIG_FB_MSM

#define MDDI_CLIENT_CORE_BASE  0x108000
#define LCD_CONTROL_BLOCK_BASE 0x110000
#define PWM_BLOCK_BASE         0x140000
#define DPSUS       (MDDI_CLIENT_CORE_BASE|0x24)
#define SYSCLKENA   (MDDI_CLIENT_CORE_BASE|0x2C)
#define START       (LCD_CONTROL_BLOCK_BASE|0x08)
#define PWM0OFF     (PWM_BLOCK_BASE|0x1C)

static void mddi0_panel_power(struct mddi_panel_info *panel, int on)
{
	/*if(on) {
		mddi_remote_write(panel->mddi, 0, DPSUS);
		udelay(122);
		mddi_remote_write(panel->mddi, 1, SYSCLKENA);
		mddi_remote_write(panel->mddi, kaiser_ffa ? 0 : 0x00001387, PWM0OFF);
	}
	else {
		mddi_remote_write(panel->mddi, kaiser_ffa ? 0x00001387 : 0, PWM0OFF);
		udelay(122);
		mddi_remote_write(panel->mddi, 0, SYSCLKENA);
		mddi_remote_write(panel->mddi, 1, DPSUS);
	}
	*/
	printk("mddi0_panel_power(%d)\n", on);
}

static struct msm_mddi_platform_data msm_mddi0_pdata = {
	.panel_power	= mddi0_panel_power,
	.has_vsync_irq	= 0,
	.fb_base = MSM_FB_BASE,
	.fb_size = MSM_FB_SIZE,
};

static struct platform_device msm_mddi0_device = {
	.name	= "msm_mddi",
	.id	= 0,
	.dev	= {
		.platform_data = &msm_mddi0_pdata
	},
};

#endif
static struct resource msm_serial0_resources[] = {
	{
		.start	= INT_UART1,
		.end	= INT_UART1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= MSM_UART1_PHYS,
		.end	= MSM_UART1_PHYS + MSM_UART1_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
};

static struct platform_device msm_serial0_device = {
	.name	= "msm_serial",
	.id	= 0,
	.num_resources	= ARRAY_SIZE(msm_serial0_resources),
	.resource	= msm_serial0_resources,
};

static struct resource usb_resources[] = {
	{
		.start	= MSM_HSUSB_PHYS,
		.end	= MSM_HSUSB_PHYS + MSM_HSUSB_SIZE,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= INT_USB_HS,
		.end	= INT_USB_HS,
		.flags	= IORESOURCE_IRQ,
	},
};

/* The HSUSB PHY on kaiser has a hardware bug where VBUS
 * interrupts can lock up the ULPI bus, causing USB to fail.
 * Disable these interrupts to avoid this issue.
 */
static int polaris_phy_init_seq[] = { 0x1D, 0x0D, 0x1D, 0x10, -1 };

static struct msm_hsusb_platform_data msm_hsusb_pdata = {
	.phy_init_seq	= polaris_phy_init_seq,
	.vendor_id	= 0x18d1,
	.product_id	= 0xd00d,
	.version	= 0x0100,
	.product_name	= "Polaris",
};

static struct platform_device msm_hsusb_device = {
	.name		= "msm_hsusb",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(usb_resources),
	.resource	= usb_resources,
	.dev		= {
		.coherent_dma_mask	= 0xffffffff,
		.platform_data		= &msm_hsusb_pdata,
	},
};

static struct android_pmem_platform_data android_pmem_pdata = {
	.name = "pmem",
	.start = MSM_PMEM_MDP_BASE,
	.size = MSM_PMEM_MDP_SIZE,
	.no_allocator = 1,
	.cached = 1,
};

static struct android_pmem_platform_data android_pmem_adsp_pdata = {
	.name = "pmem_adsp",
	.start = MSM_PMEM_ADSP_BASE,
	.size = MSM_PMEM_ADSP_SIZE,
	.no_allocator = 0,
	.cached = 0,
};

static struct android_pmem_platform_data android_pmem_gpu0_pdata = {
        .name = "pmem_gpu0",
        .start = MSM_PMEM_GPU0_BASE,
        .size = MSM_PMEM_GPU0_SIZE,
        .no_allocator = 1,
        .cached = 0,
};

static struct android_pmem_platform_data android_pmem_gpu1_pdata = {
        .name = "pmem_gpu1",
        .start = MSM_PMEM_GPU1_BASE,
        .size = MSM_PMEM_GPU1_SIZE,
        .no_allocator = 1,
        .cached = 0,
};

static struct platform_device android_pmem_device = {
	.name = "android_pmem",
	.id = 0,
	.dev = { .platform_data = &android_pmem_pdata },
};

static struct platform_device android_pmem_adsp_device = {
	.name = "android_pmem",
	.id = 1,
	.dev = { .platform_data = &android_pmem_adsp_pdata },
};

static struct platform_device android_pmem_gpu0_device = {
	.name = "android_pmem",
	.id = 2,
	.dev = { .platform_data = &android_pmem_gpu0_pdata },
};

static struct platform_device android_pmem_gpu1_device = {
	.name = "android_pmem",
	.id = 3,
	.dev = { .platform_data = &android_pmem_gpu1_pdata },
};

static struct gpio_event_direct_entry polaris_keymap[] = {
  {.gpio=19 ,.code=0x69},  // Wheel left = Left
  {.gpio=20 ,.code=0x74},  // Power button = power
  /*  {.gpio=21 ,.code=38}, */  // USB cable
  /* {.gpio=29 ,.code=0xe7},*/   // Camera button (mapped by vogue-ts ?)
  {.gpio=31 ,.code=0x6a}, // Wheel rotate
  {.gpio=35,.code=0x67}, // Voice recorder button
  {.gpio=37,.code=0x6c},   // Wheel down = down
  {.gpio=38,.code=0x67},   // Wheel up = Up
  {.gpio=40,.code=0x9e},   // Dial / hangup key
  {.gpio=41,.code=0xe5}, // Explorer key = menu
  {.gpio=42,.code=0x6b}, // Tap screen (mapped by vogue-ts ?)
  {.gpio=76 ,.code=38}, // Camera button full press
  {.gpio=90 ,.code=0xe8}, // Wheel button = ok
  {.gpio=91 ,.code=0x6a}, // Wheel right  = right
};


static struct gpio_event_input_info polaris_event_info = {
	.info.func	= gpio_event_input_func,
	.keymap		= polaris_keymap,
	.keymap_size	= ARRAY_SIZE(polaris_keymap),
	.debounce_time.tv.nsec = 1 * NSEC_PER_MSEC,
	.poll_time.tv.nsec = 20 * NSEC_PER_MSEC,
	// Debug
	//	.flags		= GPIOEDF_PRINT_KEYS,
	.flags = 0,
	.type=EV_KEY,
};

struct gpio_event_info *polaris_button_info[] = {
	&polaris_event_info.info
};

static struct gpio_event_platform_data polaris_button_data = {
	.name		= "polaris_keypad",
	.info		= polaris_button_info,
	.info_count	= ARRAY_SIZE(polaris_button_info)
};

static struct platform_device polaris_button_device = {
        .name           = GPIO_EVENT_DEV_NAME,
        .id             = -1,
        .num_resources  = 0,
        .dev            = {
                .platform_data  = &polaris_button_data,
        }
};

static struct platform_device *devices[] __initdata = {
#if !defined(CONFIG_MSM_SERIAL_DEBUGGER)
//	&msm_serial0_device,
#endif
#ifdef CONFIG_FB_MSM
	&polaris_button_device,
	&msm_mddi0_device,
#endif
//	&msm_hsusb_device,
	&android_pmem_device,
	&android_pmem_adsp_device,
	&android_pmem_gpu0_device,
	&android_pmem_gpu1_device,
};

static struct i2c_board_info __initdata polaris_i2c_board_info[] = {
        {
                I2C_BOARD_INFO("vogue-ts", 0x48),
                .type           = "vogue-ts",
		.irq            = MSM_GPIO_TO_INT(99),
        },
};


static unsigned int polaris_sdcc_slot_status(struct device *dev)
{
	int slot=1-gpio_get_value(94);
	//  printk("sdcc_slot_status=%d\n",slot);
	return slot;
}

static struct mmc_platform_data polaris_sdcc_data = {
	.ocr_mask	= MMC_VDD_28_29,
	.status		= polaris_sdcc_slot_status,
};

extern struct sys_timer msm_timer;

static void __init polaris_init_irq(void)
{
	msm_init_irq();
}

static struct msm_clock_platform_data polaris_clock_data = {
	.acpu_switch_time_us = 50,
	.max_speed_delta_khz = 256000,
	.vdd_switch_time_us = 62,
};

void msm_serial_debug_init(unsigned int base, int irq, 
			   const char *clkname, int signal_irq);

static void __init polaris_init_mmc(void)
{
	
	struct vreg *vreg_mmc;
	int rc;

	vreg_mmc = vreg_get(0, "mmc");
	rc = vreg_enable(vreg_mmc);
	if (rc)
		printk(KERN_ERR "%s: vreg enable failed (%d)\n", __func__, rc);

	// This is for SD card
	msm_add_sdcc(2, &polaris_sdcc_data);

/* TODO: Enable these once we have support for the SDC mux
 *	msm_add_sdcc(2, &polaris_sdcc_data);
 *	msm_add_sdcc(3, &polaris_sdcc_data);
 *	msm_add_sdcc(4, &polaris_sdcc_data);
 */

}

static void __init polaris_init(void)
{
#if defined(CONFIG_MSM_SERIAL_DEBUGGER)
	msm_serial_debug_init(MSM_UART1_PHYS, INT_UART1,
			      "uart1_clk", 1);
#endif
	msm_init_gpio();
	platform_add_devices(devices, ARRAY_SIZE(devices));
	msm_add_devices();
	polaris_init_mmc();
	i2c_register_board_info(0, polaris_i2c_board_info,
                ARRAY_SIZE(polaris_i2c_board_info));
	/* TODO: detect vbus and correctly notify USB about its presence 
	 * For now we just declare that VBUS is present at boot and USB
	 * copes, but this is not ideal.
	 */
	//msm_hsusb_set_vbus_state(1);
}

static void __init polaris_map_io(void)
{
	msm_map_common_io();
	msm_clock_init(&polaris_clock_data);
}

MACHINE_START(HTC_POLARIS, "HTC Polaris")
/* Maintainer: Patrick Remy*/
        /* UART for LL DEBUG */
	.phys_io	= MSM_UART1_PHYS,
	.io_pg_offst	= ((MSM_UART1_BASE) >> 18) & 0xfffc,

	.boot_params	= 0x10000100,
	.map_io		= polaris_map_io,
	.init_irq	= polaris_init_irq,
	.init_machine	= polaris_init,
	.timer		= &msm_timer,
MACHINE_END
