/* linux/arch/arm/mach-msm/board-htcvogue.c
**
**Author: Martin Johnson <m.j.johnson@massey.ac.nz>
**Based on board-halibut.c by Brian Swetland <swetland@google.com>
**
** This software is licensed under the terms of the GNU General Public
** License version 2, as published by the Free Software Foundation, and
** may be copied, distributed, and modified under those terms.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
*/

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/input.h>
#include <linux/i2c.h>


#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>

#include <asm/arch/board.h>
#include <asm/arch/clock.h>

#include <asm/io.h>
#include <asm/delay.h>

#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>

#define DECLARE_MSM_IOMAP
#include <asm/arch/msm_iomap.h>
#include <linux/gpio_event.h>

#define MSM_SMI_BASE		0x00000000
#define MSM_SMI_SIZE		0x900000

#define MSM_EBI_BASE		0x10000000
#define MSM_EBI_SIZE		0x6e00000

#define MSM_PMEM_GPU0_BASE	0x0
#define MSM_PMEM_GPU0_SIZE	0x800000

#define MSM_LINUX_BASE		MSM_EBI_BASE
#define MSM_LINUX_SIZE		0x4c00000

#define MSM_PMEM_MDP_BASE	MSM_LINUX_BASE + MSM_LINUX_SIZE
#define MSM_PMEM_MDP_SIZE	0x800000

#define MSM_PMEM_ADSP_BASE	MSM_PMEM_MDP_BASE + MSM_PMEM_MDP_SIZE
#define MSM_PMEM_ADSP_SIZE	0x800000

#define MSM_PMEM_GPU1_BASE	MSM_PMEM_ADSP_BASE + MSM_PMEM_ADSP_SIZE
#define MSM_PMEM_GPU1_SIZE	0x800000

#define MSM_FB_BASE		MSM_PMEM_GPU1_BASE + MSM_PMEM_GPU1_SIZE
#define MSM_FB_BASE  	 	0x00000000
#define MSM_FB_SIZE		0x9b000


void msm_init_irq(void);
void msm_init_gpio(void);
void msm_pmrr_config(void);


static void mddi0_panel_power(int on) {
	printk("mddi0_panel_power(%d)\n", on);
}

static struct msm_mddi_platform_data msm_mddi0_pdata = {
	.panel_power = mddi0_panel_power,
	.has_vsync_irq = 1,
	.fb_base = MSM_FB_BASE,
	.fb_size = MSM_FB_SIZE,
};

static struct platform_device msm_mddi0_device = {
	.name = "msm_mddi",
	.id = 0,
	.dev = { .platform_data = &msm_mddi0_pdata },
};
/*
static struct platform_device msm_serial0_device = {
	.name = "msm_serial",
	.id = 0,
};
static struct platform_device msm_mddi_panel = {
	.name = "mddi_panel",
	.id = 0,
};
*/
static struct i2c_board_info __initdata vogue_i2c_board_info[] = {
        {
                I2C_BOARD_INFO("tsc2003-ts", 0x48),
                .type           = "tsc2003-ts",
		        .irq            = MSM_GPIO_TO_INT(99),
        },
};

static struct resource usb_resources[] = {
	{
		.start	= MSM_HSUSB_PHYS,
		.end	= MSM_HSUSB_PHYS + MSM_HSUSB_SIZE,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= INT_USB_HS,
		.end	= INT_USB_HS,
		.flags	= IORESOURCE_IRQ,
	},
};
static int halibut_phy_init_seq[] = {  0x1D, 0x0D, 0x1D, 0x10, -1 };

static struct msm_hsusb_platform_data msm_hsusb_pdata = {
	.phy_init_seq	= halibut_phy_init_seq,
	.vendor_id	= 0xfad,
	.product_id	= 0xed,
	.version	= 0x010,
	.product_name	= "Vogue",
};

static struct platform_device msm_hsusb_device = {
	.name		= "msm_hsusb",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(usb_resources),
	.resource	= usb_resources,
	.dev		= {
		.coherent_dma_mask	= 0xffffffff,
		.platform_data		= &msm_hsusb_pdata,
	},
};

static struct gpio_event_direct_entry vogue_keymap[] = {
	// I got these from the emulator
	{.gpio=31,.code=0x6a}, // Right
	{.gpio=32,.code=0x69}, // Left
	{.gpio=33,.code=0xe8}, // OK
	{.gpio=34,.code=0x6c}, // Up
	{.gpio=35,.code=0x67}, // Down
	{.gpio=41,.code=0xe5}, // Menu
	{.gpio=42,.code=0x9e}, // Back
};


static struct gpio_event_input_info vogue_event_info = {
	.info.func	= gpio_event_input_func,
	.keymap		= vogue_keymap,
	.keymap_size	= ARRAY_SIZE(vogue_keymap),
	.debounce_time.tv.nsec = 1 * NSEC_PER_MSEC,
	.poll_time.tv.nsec = 20 * NSEC_PER_MSEC,
	.flags		= 0,
	.type=EV_KEY,
};

struct gpio_event_info *vogue_button_info[] = {
	&vogue_event_info.info
};

static struct gpio_event_platform_data vogue_button_data = {
	.name		= "vogue_keypad",
	.info		= vogue_button_info,
	.info_count	= ARRAY_SIZE(vogue_button_info)
};

static struct platform_device vogue_button_device = {
        .name           = GPIO_EVENT_DEV_NAME,
        .id             = -1,
        .num_resources  = 0,
        .dev            = {
                .platform_data  = &vogue_button_data,
        }
};


static struct platform_device *devices[] __initdata = {
//	&htcvogue_fb,
	&msm_mddi0_device,
		//	&msm_hsusb_device,
	&vogue_button_device,
//	&rndis,
/*
**	&msm_serial0_device,
**	&halibut_keypad_device,
**	&msm_mddi0_device,
**	&smc91x_device,
*/
};


extern struct sys_timer msm_timer;

static void __init htcvogue_init_irq(void)
{
	msm_init_irq();
}
static struct msm_clock_platform_data vogue_clock_data = {
	.acpu_switch_time_us = 50,
	.max_speed_delta_khz = 256000,
	.vdd_switch_time_us = 62,
};


static void __init htcvogue_init(void)
{
	msm_init_gpio();
	platform_add_devices(devices, ARRAY_SIZE(devices));
	msm_add_devices();
	i2c_register_board_info(0, vogue_i2c_board_info,
                        ARRAY_SIZE(vogue_i2c_board_info));

	//	msm_hsusb_set_vbus_state(1);
}

static void __init htcvogue_map_io(void)
{
	msm_map_common_io();
}

MACHINE_START(HTCVOGUE, "HTC Vogue")
/* Maintainer: Martin Johnson */

	.phys_io        = 0x0,
	.io_pg_offst    = 0x0,
	
	.boot_params    = PHYS_OFFSET+0x100,
	.map_io         = htcvogue_map_io,
	.init_irq       = htcvogue_init_irq,
	.init_machine   = htcvogue_init,
	.timer          = &msm_timer,
MACHINE_END
