/*
Vogue Touch Screen Driver by Martin Johnson <M.J.Johnson@massey.ac.nz>
Based on TSC2003 driver by Bill Gatliff
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/i2c.h>
#include <linux/string.h>
#include <linux/bcd.h>
#include <linux/list.h>
#include <linux/device.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/input.h>
#include <linux/platform_device.h>
#include <asm/delay.h>
#include <linux/delay.h>
#include <asm/io.h>
#include <linux/fb.h>
#include <linux/font.h>
#include <asm/arch/msm_fb.h>
#include <asm/arch/onscreen_keyboard.h>
#include "proc_comm.h"
#include <asm/arch/gpio.h>
#include <asm/arch/msm_iomap.h>

#define DRIVER_NAME "vogue-ts"

extern void msm_update_screen(void);

int onscreen_keyboard=0;
int keyboard_drawn=0;

#define NKEYS 50
int key_codes[NKEYS]={
KEY_1,KEY_2,KEY_3,KEY_4,KEY_5,KEY_6,KEY_7,KEY_8,KEY_9,KEY_0,KEY_Q,KEY_W,KEY_E,KEY_R,
KEY_T,KEY_Y,KEY_U,KEY_I,KEY_O,KEY_P,KEY_A,KEY_S,KEY_D,KEY_F,KEY_G,KEY_H,KEY_J,KEY_K,
KEY_L,KEY_BACKSPACE,KEY_LEFTSHIFT,KEY_Z,KEY_X,KEY_C,KEY_V,KEY_B,KEY_N,KEY_M,KEY_DOT,KEY_ENTER,
KEY_RIGHTALT,KEY_COMPOSE,KEY_LEFTCTRL,KEY_SPACE,KEY_SPACE,KEY_SPACE,KEY_SPACE,KEY_SLASH,KEY_COMMA,-1
};
#include "android_keyboard.c"
#include "android_keyboard_big.c"
short keyboard_image[KEYBOARD_HEIGHT][240];
short big_keyboard_image[KEYBOARD_HEIGHT*2][480];;
struct osk_type vogue_keys[62];
int keyboard_loaded=0;
void load_keyboard_image(void) {
	int i;
	short *s=(short *)keyboard_image;
	for(i=0;i<gimp_image.height*gimp_image.width*3;i+=3)
		*s++=((gimp_image.pixel_data[i]>>3)<<11) | 
			((gimp_image.pixel_data[i+1]>>2)<<5) |
			(gimp_image.pixel_data[i+2]>>3);
	for(i=0;i<NKEYS;i++) {
		vogue_keys[i].key=key_codes[i];
		vogue_keys[i].x=(i%10)*24;
		vogue_keys[i].y=((i/10)*232)/10+(320-KEYBOARD_HEIGHT);
		vogue_keys[i].w=24;
		vogue_keys[i].h=23;
	}
	s=(short *)big_keyboard_image;
	for(i=0;i<big_kbd.height*big_kbd.width*3;i+=3)
		*s++=((big_kbd.pixel_data[i]>>3)<<11) | 
			((big_kbd.pixel_data[i+1]>>2)<<5) |
			(big_kbd.pixel_data[i+2]>>3);
	keyboard_loaded=1;
}

extern short* msm_display_base;


void draw_keyboard(void) {
	short *kbd=(short *)keyboard_image;
	short *disp;
	int i;

	if(!msm_display_base) return;
	if(!keyboard_loaded)
		load_keyboard_image();

	disp=msm_display_base+640*240;
	for(i=0;i<KEYBOARD_HEIGHT*240;i++)
		disp[i]=kbd[i];
	keyboard_drawn=1;
}

enum ts_pd {
	PD_POWERDOWN = 0, /* penirq */
	PD_IREFOFF_ADCON = 1, /* no penirq */
	PD_IREFON_ADCOFF = 2, /* penirq */
	PD_IREFON_ADCON = 3, /* no penirq */
	PD_PENIRQ_ARM = PD_IREFON_ADCOFF,
	PD_PENIRQ_DISARM = PD_IREFON_ADCON,
};

enum ts_m {
	M_12BIT = 0,
	M_8BIT = 1
};

enum ts_cmd {
	MEAS_TEMP0 = 0,
	MEAS_VBAT1 = 1,
	MEAS_IN1 = 2,
	MEAS_TEMP1 = 4,
	MEAS_VBAT2 = 5,
	MEAS_IN2 = 6,
	ACTIVATE_NX_DRIVERS = 8,
	ACTIVATE_NY_DRIVERS = 9,
	ACTIVATE_YNX_DRIVERS = 10,
	MEAS_XPOS = 12,
	MEAS_YPOS = 13,
	MEAS_Z1POS = 14,
	MEAS_Z2POS = 15
};

#define TS_CMD(cn,pdn,m) (((cn) << 4) | ((pdn) << 2) | ((m) << 1))

#define ADC_MAX ((1 << 12) - 1)


struct cpld {
	short v[8];
};

struct cpld *vogue_cpld;

void set_egpio(int no,int bit, int value) {
	short oldval;
	oldval=vogue_cpld->v[no] & 0xff;
	if(value)
		oldval=oldval|(1<<bit);
	else
		oldval=oldval&(~(1<<bit));
	vogue_cpld->v[no]=oldval;
}

int get_egpio(int no,int bit) {
	if(vogue_cpld->v[no] & (1<<bit))
		return 1;
	else return 0;
}

int range(int x, int min, int max) {
	if(x<min) return min;
	if(x>max) return max;
	return x;
}

struct ts_data {
	struct i2c_client *client;
	struct input_dev *idev;
	struct timer_list penirq_timer;
	struct semaphore sem;
	struct task_struct *tstask;
	struct completion tstask_completion;
	enum ts_pd pd;
	enum ts_m m;
	int vbat1;
	int vbat2;
	int temp0;
	int temp1;
	int in1;
	int in2;
};

static inline int ts_command (struct ts_data *data,
	enum ts_cmd cmd,
	enum ts_pd pd)
{
	char c;
	int ret;
	down(&data->sem);
	c = TS_CMD(cmd, pd, data->m);
	ret = i2c_master_send(data->client, &c, 1);
	up(&data->sem);
	return ret;
}

static int ts_read (struct ts_data *data,
	enum ts_cmd cmd,
	enum ts_pd pd,
	int *val)
{
	char c;
	char d[2];
	int ret;

	c = TS_CMD(cmd, pd, data->m);
	ret = i2c_master_send(data->client, &c, 1);

	udelay(20);
	ret = i2c_master_recv(data->client, d, data->m == M_12BIT ? 2 : 1);

	if (val)
	{
		*val = d[0];
		*val <<= 4;
		if (data->m == M_12BIT)
			*val += (d[1] >> 4);
	}

#if defined(CONFIG_I2C_DEBUG_CHIP)
	printk(KERN_ERR "%s: val[%x] = %d\n",
	       __FUNCTION__, cmd, (((int)d[0]) << 8) + d[1]);
#endif

	return 0;
	if (!ret) ret = -ENODEV;
	return ret;
}

static inline int ts_read_temp0 (struct ts_data *d, enum
				      ts_pd pd, int *t)
{
	return ts_read(d, MEAS_TEMP0, pd, t);
}

static inline int ts_read_temp1 (struct ts_data *d, enum
				      ts_pd pd, int *t)
{
	return ts_read(d, MEAS_TEMP1, pd, t);
}

static inline int ts_read_xpos (struct ts_data *d, enum
				     ts_pd pd, int *x)
{
	return ts_read(d, MEAS_XPOS, pd, x);
}

static inline int ts_read_ypos (struct ts_data *d, enum
				     ts_pd pd, int *y)
{
	return ts_read(d, MEAS_YPOS, pd, y);
}

static inline int ts_read_pressure (struct ts_data *d, enum
					 ts_pd pd, int *p)
{
	return ts_read(d, MEAS_Z1POS, pd, p);
}

static inline int ts_read_p2 (struct ts_data *d, enum
					 ts_pd pd, int *p)
{
	return ts_read(d, MEAS_Z2POS, pd, p);
}

static inline int ts_read_in1 (struct ts_data *d, enum
				    ts_pd pd, int *t)
{
	return ts_read(d, MEAS_IN1, pd, t);
}

static inline int ts_read_in2 (struct ts_data *d, enum
				    ts_pd pd, int *t)
{
	return ts_read(d, MEAS_IN2, pd, t);
}

static inline int ts_read_vbat1 (struct ts_data *d, enum
				      ts_pd pd, int *t)
{
	return ts_read(d, MEAS_VBAT1, pd, t);
}

static inline int ts_read_vbat2 (struct ts_data *d, enum
				      ts_pd pd, int *t)
{
	return ts_read(d, MEAS_VBAT2, pd, t);
}

static inline int ts_powerdown (struct ts_data *d)
{
	/* we don't have a distinct powerdown command,
	   so do a benign read with the PD bits cleared */
	return ts_read(d, MEAS_IN1, PD_POWERDOWN, 0);
}
void vibrate(int i) {
	unsigned data1 = 0;
#ifdef CONFIG_MACH_HTCVOGUE
	unsigned cmd_on = 0x4;
	unsigned cmd_off = 0x5;
	unsigned data2 = 0xb22;
#endif
#ifdef CONFIG_MACH_HTCPOLARIS
	unsigned cmd_on = 0x17;
	unsigned cmd_off = 0x18;
	unsigned data2 = 0;
#endif

	if(i)
		msm_proc_comm(cmd_on, &data1, &data2);
	else
		msm_proc_comm(cmd_off, &data1, &data2);
}

void vibrate_time(int t) {
	vibrate(1);
	schedule_timeout_interruptible(msecs_to_jiffies(t));
	vibrate(0);
}


void ts_init_client (struct i2c_client *client)
{
	struct ts_data *data = i2c_get_clientdata(client);

	data->pd = PD_PENIRQ_DISARM;
	data->m = M_8BIT;
	return;
}
#ifdef CONFIG_MACH_HTCVOGUE
int XMIN=735;
int YMIN=367;
#endif
#ifdef CONFIG_MACH_HTCPOLARIS
int XMIN=457;
int YMIN=484;
#endif
int XMAX=3487;
int YMAX=3775;

struct keymap {
	int no;
	int bit;
	int key;
	int val;
};
#ifdef CONFIG_MACH_HTCVOGUE
static struct keymap keys[3]={{6,2,0xe7,1},
			      {6,3,0x6b,1},
			      {5,7,-1,1}};
#endif
#ifdef CONFIG_MACH_HTCPOLARIS
static struct keymap keys[3]={{1,26,0xe7,1}, // 2*16+10 = 42 = tap screen
			      {6,3,0x6b,1},  // Todo: check whether we need this
			      {1,13,-1,1}};  // 1*16+13 = 29 = cam button
#endif
static int tsts_thread (void *v)
{
	struct ts_data *d = v;
	struct task_struct *tsk = current;
	int pendown=0;
	int thiskey=0;
	int shift=0;	
	int ctrl=0;
	int rightalt=0;
	int vibrating=0;

	d->tstask = tsk;

	daemonize(DRIVER_NAME "tsd");
	allow_signal(SIGKILL);

	complete(&d->tstask_completion);

	printk(KERN_INFO "%s: address 0x%x\n",
	       __FUNCTION__, d->client->addr);

	while (!signal_pending(tsk))
	{
	  int x, y, p, p1, p2, i, a, b, kx, ky, vx, vy;
	  short *disp;

		ts_read_xpos(d, PD_PENIRQ_DISARM, &x);
		ts_read_ypos(d, PD_PENIRQ_DISARM, &y);
		ts_read_pressure(d, PD_PENIRQ_DISARM, &p1);
		ts_read_p2(d, PD_PENIRQ_DISARM, &p2);
		if(p1==0)
			p=0; 
		else
			p=512-((((x*256)/4096)* (((p2*256)/p1)-256))>>8);
		if(p<0) p=0;
		if(p>512) p=512;
		//		printk("(%d,%d,%d,%d,%d) pendown=%d\n",x,y,p,p1,p2,pendown);
		
		if (p > 10 && x>0 && y>0 && x<ADC_MAX-1 && y<ADC_MAX-1) {
		  //		  printk("(%d,%d,%d,%d,%d) pendown=%d shift=%d ctrl=%d thiskey=%d\n",x,y,p,p1,p2,pendown,shift,ctrl,thiskey);
			x=ADC_MAX-x;
			y=ADC_MAX-y;
			if(keys[2].val==0 && onscreen_keyboard) {	// do calibration when cam button is down and kbd displayed
				if(x<ADC_MAX/2) XMIN=x;
				else	XMAX=x;
				if(y<ADC_MAX/2) YMIN=y;
				else	YMAX=y;
			}
			if(x<XMIN) x=XMIN; //XMIN=x;
			if(x>XMAX) x=XMAX; //XMAX=x;
			if(y<YMIN) y=YMIN; //YMIN==y;
			if(y>YMAX) y=YMAX; //YMAX=y;

			x=((x-XMIN)*240)/(XMAX-XMIN);
			y=((y-YMIN)*320)/(YMAX-YMIN);
			if(onscreen_keyboard && y>=320-KEYBOARD_HEIGHT) {
				for(i=0;i<NKEYS;i++)
					if(x>=vogue_keys[i].x && x<vogue_keys[i].x+vogue_keys[i].w &&
					   y>=vogue_keys[i].y && y<vogue_keys[i].y+vogue_keys[i].h) {
						   //						   if(pendown==0) {
							if(vogue_keys[i].key==-1) {
								if(pendown==0) {
									if(onscreen_keyboard==1)
										onscreen_keyboard=2;
									else
										onscreen_keyboard=1;
									msm_update_screen();
								}
							} else {
								if(shift)
									 input_report_key(d->idev,KEY_LEFTSHIFT,1);
								if(ctrl)
									input_report_key(d->idev,KEY_LEFTCTRL,1);
								if(rightalt)
									input_report_key(d->idev,KEY_RIGHTALT,1);

								input_report_key(d->idev,vogue_keys[i].key,1);
								input_sync(d->idev);
								if(pendown==0) {
									vibrating=1;

								}
							}
						   //}
							draw_keyboard();
							thiskey=vogue_keys[i].key;
							vx=vogue_keys[i].x;
							kx=range(vx-12,0,240-48);
							vy=vogue_keys[i].y-(320-KEYBOARD_HEIGHT);
							ky=range(vy-12,0,KEYBOARD_HEIGHT-48);
							disp=msm_display_base+640*240;
							for(a=0;a<48;a++)
							  for(b=0;b<48;b++)
							    disp[(ky+a)*240+kx+b]=big_keyboard_image[vy*2+a][vogue_keys[i].x*2+b];
							msm_update_screen();
						   
						   break;
					   }
			} else {
				if(onscreen_keyboard==2) y=y+KEYBOARD_HEIGHT;
				input_report_abs(d->idev, ABS_X, x);
				input_report_abs(d->idev, ABS_Y, y);
				input_report_abs(d->idev, ABS_PRESSURE, p);
				input_report_key(d->idev, BTN_TOUCH, 1);
				input_sync(d->idev);
			}
			pendown = 1;
			
		} else if (pendown == 1) {
		  //		  printk("(%d,%d,%d,%d,%d) pendown=%d shift=%d ctrl=%d thiskey=%d\n",x,y,p,p1,p2,pendown,shift,ctrl,thiskey);
			if(onscreen_keyboard) {
				if(shift)
					input_report_key(d->idev,KEY_LEFTSHIFT,0);
				if(ctrl)
					input_report_key(d->idev,KEY_LEFTCTRL,0);
				if(rightalt)
					input_report_key(d->idev,KEY_RIGHTALT,0);
				if((thiskey!=KEY_LEFTSHIFT)&& (thiskey!=KEY_LEFTCTRL)&&(thiskey!=KEY_RIGHTALT)){
				  shift=0;
				  ctrl=0;
				  rightalt=0;
				}
				if(thiskey>0) {
					input_report_key(d->idev,thiskey,0);
					//					vibrate(0);
				}
				draw_keyboard();
				msm_update_screen();
				if(thiskey==KEY_LEFTSHIFT)
					shift=1;
				if(thiskey==KEY_LEFTCTRL)
					ctrl=1;
				if(thiskey==KEY_RIGHTALT)
					rightalt=1;
			}
			pendown = 0;
			input_report_key(d->idev, BTN_TOUCH, 0);
			input_report_abs(d->idev, ABS_PRESSURE, 0);
			
			input_sync(d->idev);
		}
		
		for(i=0;i<3;i++) {
		  int t = gpio_get_value((keys[i].no)*16+keys[i].bit);
			if(t != keys[i].val) {
				keys[i].val=t;
				if(keys[i].key==-1) {// Cam Button
					draw_keyboard();
					if(t==0) {
						if(!onscreen_keyboard) {
							onscreen_keyboard=1;
						}
						else {
							onscreen_keyboard=0;
						}
						msm_update_screen();
					} else {
					}
				}
				else {
					input_report_key(d->idev, keys[i].key,1-t);
					input_sync(d->idev);
				}
			}
		}
		
		if(vibrating==1)
			vibrate(1);
		schedule_timeout_interruptible(msecs_to_jiffies(30));
		if(vibrating) {
			vibrating--;
			if(vibrating==0)
				vibrate(0);
		}
	}

	d->tstask = NULL;
	complete_and_exit(&d->tstask_completion, 0);
}

static int ts_idev_open (struct input_dev *idev)
{
	struct ts_data *d = idev->private;
	int ret = 0;

	if (down_interruptible(&d->sem))
		return -EINTR;

	if (d->tstask)
		panic(DRIVER_NAME "tsd already running (!). abort.");

	init_completion(&d->tstask_completion);

	ret = kernel_thread(tsts_thread, d, CLONE_KERNEL);
	if (ret >= 0) {
		wait_for_completion(&d->tstask_completion);
		ret = 0;
	}

	up(&d->sem);

	return ret;
}

static void ts_idev_close (struct input_dev *idev)
{
	struct ts_data *d = idev->private;
	down_interruptible(&d->sem);
	if (d->tstask)
	{
		send_sig(SIGKILL, d->tstask, 1);
		wait_for_completion(&d->tstask_completion);
	}
	up(&d->sem);
	return;
}

static int ts_driver_register (struct ts_data *data)
{
	int ret = 0;
	int i;

	init_MUTEX(&data->sem);

	data->idev = input_allocate_device();
	if(!data->idev)
	{
		return -ENOMEM;
	}
	data->idev->private = data;
	data->idev->name = DRIVER_NAME;
	set_bit(EV_SYN, data->idev->evbit);
	set_bit(EV_KEY, data->idev->evbit);
	set_bit(BTN_TOUCH, data->idev->keybit);
	for(i=0;i<3;i++)
		if(keys[i].key>0)
			set_bit(keys[i].key, data->idev->keybit);
	for(i=0;i<NKEYS;i++)
		if(key_codes[i]>0)
			set_bit(key_codes[i], data->idev->keybit);
	set_bit(EV_ABS, data->idev->evbit);
	
	data->idev->open = ts_idev_open;
	data->idev->close = ts_idev_close;
	data->idev->absbit[ABS_X] = BIT(ABS_X);
	data->idev->absbit[ABS_Y] = BIT(ABS_Y);
	data->idev->absbit[ABS_PRESSURE] = BIT(ABS_PRESSURE);// 6799
	input_set_abs_params(data->idev, ABS_X, 0, 240, 0, 0);
	input_set_abs_params(data->idev, ABS_Y, 0, 320, 0, 0);

	ret = input_register_device(data->idev);
	if(ret)
	{
		input_free_device(data->idev);
		return ret;
	}
	return ret;
}

static int  vogue_ts_probe(struct i2c_client *client)
{
	int ret;
	struct ts_data *data;

	printk(KERN_INFO "Vogue i2c touch screen\n");
//	load_keyboard_image();

	vogue_cpld=ioremap_nocache(0x90000000,0x10);
	
	if (!i2c_check_functionality(client->adapter, I2C_FUNC_I2C)) {
		printk(KERN_ERR "vogue_ts_probe: need I2C_FUNC_I2C\n");
		ret = -ENODEV;
		goto err;
	}
	data = kcalloc(1, sizeof(*data), GFP_KERNEL);
	if (!data) {
		ret = -ENOMEM;
		goto err;
	}
	
	data->client = client;
	i2c_set_clientdata(client, data);

	ts_init_client(client);

	ts_powerdown(data);

	ret = ts_driver_register(data);
	if(ret < 0) {
		printk("driver_register failed\n");
		goto err;
	}

	return ret;

err:
	kfree(client);
	return ret;
}

#define vogue_ts_suspend NULL
#define vogue_ts_resume NULL


static int __devexit vogue_ts_remove(struct i2c_client *client)
{
	struct ts_data *ts = i2c_get_clientdata(client);
	input_unregister_device(ts->idev);
	kfree(ts);
	return 0;
}

static struct i2c_driver vogue_ts_driver = {
	.probe		= vogue_ts_probe,
	.remove		= vogue_ts_remove,
	.suspend	= vogue_ts_suspend,
	.resume		= vogue_ts_resume,
	.driver = {
		.name	= "vogue-ts",
	},
};

static int __devinit vogue_ts_init(void)
{
	printk("vogue ts_init\n");
	return i2c_add_driver(&vogue_ts_driver);
}

static void __exit vogue_ts_exit(void)
{
	i2c_del_driver(&vogue_ts_driver);
}

MODULE_AUTHOR("Martin Johnson <>");
MODULE_DESCRIPTION("Vogue Touch Screen");
MODULE_LICENSE("GPL");

module_init(vogue_ts_init);
module_exit(vogue_ts_exit);
