/* drivers/rtc/rtc-msm7x00.c
 *
 * Author: Martin Johnson <M.J.Johnson@massey.ac.nz>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <linux/module.h>
#include <linux/version.h>

#include <linux/kernel.h>
#include <linux/platform_device.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/rtc.h>

#define PDEV_NAME "msm_rtc"

#define SECSFROM_1970_TO_1980 315532800

struct rtc_device *rtc;

int msm_proc_comm(unsigned cmd, unsigned *data1, unsigned *data2);

static int
msmrtc_pmlib_set_time(struct device *dev, struct rtc_time *tm)
{
	unsigned long unix_time;

	if (rtc_valid_tm(tm))
		return -EINVAL;

	rtc_tm_to_time(tm, &unix_time);
	unix_time=unix_time-SECSFROM_1970_TO_1980; // MSM RTC starts 10 years after unix time

	msm_proc_comm(0x182,&unix_time,NULL);
	
	return 0;
}

static int
msmrtc_pmlib_read_time(struct device *dev, struct rtc_time *tm)
{
	unsigned long secs;
	unsigned retval=0;
	
	msm_proc_comm(0x181,&retval,NULL);
	
	secs=retval+SECSFROM_1970_TO_1980;	// MSM RTC starts 10 years after unix time
	rtc_time_to_tm(secs, tm);
	return 0;
}

static struct rtc_class_ops msm_rtc_ops = {
	.read_time	= msmrtc_pmlib_read_time,
	.set_time	= msmrtc_pmlib_set_time,
};

static int
msmrtc_probe(struct platform_device *pdev)
{
	printk("RTC probe\n");
	rtc = rtc_device_register("msm_rtc",
				  &pdev->dev,
				  &msm_rtc_ops,
				  THIS_MODULE);
	if (IS_ERR(rtc)) {
		printk(KERN_ERR "%s: Can't register RTC device (%ld)\n",
		       pdev->name, PTR_ERR(rtc));
		return PTR_ERR(rtc);
	}
	printk("MSM RTC started\n");
	return 0;
}


static struct platform_driver msmrtc_driver = {
	.probe	= msmrtc_probe,
	.driver	= {
		.name	= "msm_rtc",
	},
};

static int __init msmrtc_init(void)
{
	printk("MSM RTC init\n");
	return platform_driver_register(&msmrtc_driver);
}

module_init(msmrtc_init);

MODULE_DESCRIPTION("RTC driver for Qualcomm MSM7x00 chipsets");
MODULE_AUTHOR("Martin Johnson <M.J.Johnson@massey.ac.nz>");
MODULE_LICENSE("GPL");
